@@ -24,49 +24,15 @@
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#include <fcntl.h>
-#include <iomanip>
-#include <random>
 #include <fstream>
-#include <filesystem>
-#include <mutex>
+#include <vector>
 
-#include <boost/program_options.hpp>
-#include <boost/algorithm/string.hpp>
-
-#include <zeep/http/daemon.hpp>
-#include <zeep/http/server.hpp>
-#include <zeep/http/html-controller.hpp>
-#include <zeep/http/rest-controller.hpp>
-#include <zeep/crypto.hpp>
-
-#include "cif++/Secondary.hpp"
-// #include "cif++/Statistics.hpp"
-#include "cif++/CifUtils.hpp"
-#include "cif++/Cif++.hpp"
-#include "cif++/Structure.hpp"
-#include "cif++/Compound.hpp"
-
-#include <zeep/json/element.hpp>
+#include <dssp.hpp>
 
+#include "tortoize.hpp"
 #include "revision.hpp"
 
-#ifdef _MSC_VER
-//MSVC stdlib.h definitions
-#define STDIN_FILENO 0
-#define STDOUT_FILENO 1
-#define STDERR_FILENO 2
-#endif
-
-namespace po = boost::program_options;
 namespace fs = std::filesystem;
-namespace ba = boost::algorithm;
-
-using mmcif::Atom;
-using mmcif::Point;
-using mmcif::Structure;
-using mmcif::Monomer;
-using mmcif::Polymer;
 
 using json = zeep::json::element;
 
@@ -361,6 +327,20 @@ std::ostream& operator<<(std::ostream& os, SecStrType ss)
 	return os;
 }
 
+std::string to_string(SecStrType ss)
+{
+	switch (ss)
+	{
+		case SecStrType::helix:		return "helix"; break;
+		case SecStrType::strand:	return "strand"; break;
+		case SecStrType::other:		return "other"; break;
+		case SecStrType::cis:		return "cis"; break;
+		case SecStrType::prepro:	return "prepro"; break;
+	}
+
+	throw std::runtime_error("Invalid sec structure");
+}
+
 // --------------------------------------------------------------------
 // The header for the data blocks as written in de resource
 
@@ -592,7 +572,7 @@ float Data::interpolatedCount(float a1, float a2) const
 	return result;
 }
 
-void buildDataFile(fs::path dir)
+void buildDataFile(const fs::path &dir)
 {
 	using namespace std::literals;
 
@@ -626,7 +606,7 @@ void buildDataFile(fs::path dir)
 	std::vector<uint8_t> bits;
 
 	// first ramachandran counts
-	for (auto aa: mmcif::kAAMap)
+	for (auto aa: cif::compound_factory::kAAMap)
 	{
 		for (std::pair<SecStrType, const char*> ss: {
 				std::make_pair(SecStrType::helix, "helix"),
@@ -681,7 +661,7 @@ void buildDataFile(fs::path dir)
 	bits.clear();
 
 	// next torsion counts
-	for (auto aa: mmcif::kAAMap)
+	for (auto aa: cif::compound_factory::kAAMap)
 	{
 		for (std::pair<SecStrType, const char*> ss: {
 				std::make_pair(SecStrType::helix, "helix"),
@@ -798,7 +778,7 @@ const Data& DataTable::loadRamachandranData(const std::string& aa, SecStrType ss
 	}
 
 	if (i == m_ramachandran.end())
-		throw std::runtime_error("Data missing for aa= " + aa + " and ss = '" + static_cast<char>(ss) + '\'');
+		throw std::runtime_error("Data missing for aa = " + aa + " and ss = '" + static_cast<char>(ss) + '\'');
 	
 	return *i;
 }
@@ -807,7 +787,7 @@ void DataTable::load(const char* name, std::vector<Data>& table, float& mean, fl
 {
 	using namespace std::literals;
 
-	auto rfd = cif::loadResource(name);
+	auto rfd = cif::load_resource(name);
 
 	if (not rfd)
 		throw std::runtime_error("Missing resource "s + name);
@@ -863,9 +843,9 @@ float jackknife(const std::vector<float>& zScorePerResidue)
 
 // --------------------------------------------------------------------
 
-json calculateZScores(const Structure& structure)
+json calculateZScores(const cif::mm::structure& structure)
 {
-	mmcif::DSSP dssp(structure, 3, false);
+	dssp dssp(structure, 3, false);
 	auto& tbl = DataTable::instance();
 
 	double ramaZScoreSum = 0;
@@ -888,22 +868,20 @@ json calculateZScores(const Structure& structure)
 			if (phi == 360 or psi == 360)
 				continue;
 
-			std::tuple<std::string,int,std::string,std::string> pdbID = structure.MapLabelToPDB(res.asymID(), res.seqID(), res.compoundID(), res.authSeqID());
+			std::string aa = res.get_compound_id();
 
 			json residue = {
-				{ "asymID", res.asymID() },
-				{ "seqID", res.seqID() },
-				{ "compID", res.compoundID() },
+				{ "asymID", res.get_asym_id() },
+				{ "seqID", res.get_seq_id() },
+				{ "compID", aa },
 				{ "pdb", {
-					{ "strandID", std::get<0>(pdbID) },
-					{ "seqNum", std::get<1>(pdbID) },
-					{ "compID", std::get<2>(pdbID) },
-					{ "insCode", std::get<3>(pdbID) }
+					{ "strandID", res.get_auth_asym_id() },
+					{ "seqNum", std::stoi(res.get_auth_seq_id()) },
+					{ "compID", aa },
+					{ "insCode", res.get_pdb_ins_code() }
 				}}
 			};
 
-			std::string aa = res.compoundID();
-
 			// remap some common modified amino acids
 			if (aa == "MSE")
 			{
@@ -932,9 +910,9 @@ json calculateZScores(const Structure& structure)
 
 				aa = "GLU";
 			}
-			else if (mmcif::kAAMap.find(aa) == mmcif::kAAMap.end())
+			else if (not cif::compound_factory::instance().kAAMap.count(aa))
 			{
-				if (cif::VERBOSE)
+				if (cif::VERBOSE > 0)
 					std::cerr << "Replacing " << aa << " with ALA" << std::endl;
 
 				aa = "ALA";
@@ -942,28 +920,28 @@ json calculateZScores(const Structure& structure)
 			
 			SecStrType tors_ss, rama_ss;
 
-			switch (dssp(res))
+			switch (dssp[{res.get_asym_id(), res.get_seq_id()}].type())
 			{
-				case mmcif::ssAlphahelix:	tors_ss = SecStrType::helix; break;
-				case mmcif::ssStrand:		tors_ss = SecStrType::strand; break;
-				default:					tors_ss = SecStrType::other; break;
+				case dssp::structure_type::Alphahelix:	tors_ss = SecStrType::helix; break;
+				case dssp::structure_type::Strand:		tors_ss = SecStrType::strand; break;
+				default:								tors_ss = SecStrType::other; break;
 			}
 
-			if (aa != "PRO" and poly[i + 1].compoundID() == "PRO")
+			if (aa != "PRO" and poly[i + 1].get_compound_id() == "PRO")
 				rama_ss = SecStrType::prepro;
-			else if (aa == "PRO" && res.isCis())
+			else if (aa == "PRO" && res.is_cis())
 				rama_ss = SecStrType::cis;
 			else
 				rama_ss = tors_ss;
 
-#pragma warning "todo"
+// TODO: #pragma warning "todo" (but the question now is, what is here to do???)
 			auto& rd = tbl.loadRamachandranData(aa, rama_ss);
 
 			auto zr = rd.zscore(phi, psi);
 
 			residue["ramachandran"] =
 			{
-				{ "ss-type", boost::lexical_cast<std::string>(rama_ss) },
+				{ "ss-type", to_string(rama_ss) },
 				{ "z-score", zr }
 			};
 
@@ -976,7 +954,7 @@ json calculateZScores(const Structure& structure)
 			{
 				float zt = nan("1");
 
-				auto chiCount = res.nrOfChis();
+				auto chiCount = res.nr_of_chis();
 				if (chiCount)
 				{
 					float chi1 = res.chi(0);
@@ -993,14 +971,14 @@ json calculateZScores(const Structure& structure)
 
 					residue["torsion"] =
 					{
-						{ "ss-type", boost::lexical_cast<std::string>(tors_ss) },
+						{ "ss-type", to_string(tors_ss) },
 						{ "z-score", zt }
 					};
 				}
 			}
 			catch (const std::exception& e)
 			{
-				if (cif::VERBOSE)
+				if (cif::VERBOSE > 0)
 					std::cerr << e.what() << '\n';
 			}
 
@@ -1024,339 +1002,9 @@ json calculateZScores(const Structure& structure)
 }
 
 // --------------------------------------------------------------------
-#if WEBSERVICE
 
-class tortoize_html_controller : public zeep::http::html_controller
+json tortoize_calculate(const fs::path &xyzin)
 {
-  public:
-	tortoize_html_controller()
-		: zeep::http::html_controller("tortoize")
-	{
-		mount("{css,scripts,fonts,images,favicon}/", &tortoize_html_controller::handle_file);
-		mount("{favicon.ico,browserconfig.xml,manifest.json}", &tortoize_html_controller::handle_file);
-		mount("", &tortoize_html_controller::index);
-	}
-
-	void index(const zeep::http::request& request, const zeep::http::scope& scope, zeep::http::reply& reply)
-	{
-		get_template_processor().create_reply_from_template("index", scope, reply);
-	}
-};
-
-// --------------------------------------------------------------------
-
-class tortoize_rest_controller : public zeep::http::rest_controller
-{
-  public:
-	tortoize_rest_controller()
-		: zeep::http::rest_controller("")
-		, m_tempdir(fs::temp_directory_path() / "tortoize-ws")
-	{
-		fs::create_directories(m_tempdir);
-
-		map_post_request("tortoize", &tortoize_rest_controller::calculate, "data", "dict");
-	}
-
-	json calculate(const std::string& file, const std::string& dict)
-	{
-		// First store dictionary, just in case
-
-		fs::path dictFile;
-
-		if (not dict.empty())\
-		{
-			dictFile = m_tempdir / ("dict-" + std::to_string(m_next_dict_nr++));
-			std::ofstream tmpFile(dictFile);
-			tmpFile << dict;
-
-			mmcif::CompoundFactory::instance().pushDictionary(dictFile);
-		}
-
-		try
-		{
-			// --------------------------------------------------------------------
-			
-			json data{
-				{ "software",
-					{
-						{ "name", "tortoize" },
-						{ "version", kVersionNumber },
-						{ "reference", "Sobolev et al. A Global Ramachandran Score Identifies Protein Structures with Unlikely Stereochemistry, Structure (2020)" },
-						{ "reference-doi", "https://doi.org/10.1016/j.str.2020.08.005" }
-					}
-				}
-			};
-
-			// --------------------------------------------------------------------
-
-			mmcif::File f(file.data(), file.length());
-
-			std::set<uint32_t> models;
-			for (auto r: f.data()["atom_site"])
-			{
-				if (not r["pdbx_PDB_model_num"].empty())
-					models.insert(r["pdbx_PDB_model_num"].as<uint32_t>());
-			}
-
-			if (models.empty())
-				models.insert(0);
-
-			for (auto model: models)
-			{
-				Structure structure(f, model);
-				data["model"][std::to_string(model)] = calculateZScores(structure);
-			}
-
-			if (not dictFile.empty())
-			{
-				mmcif::CompoundFactory::instance().popDictionary();
-				fs::remove(dictFile);
-			}
-
-			return data;
-		}
-		catch (...)
-		{
-			std::error_code ec;
-
-			if (not dictFile.empty())
-			{
-				mmcif::CompoundFactory::instance().popDictionary();
-				fs::remove(dictFile);
-			}
-
-			throw;
-		}
-	}
-
-	fs::path m_tempdir;
-	size_t m_next_dict_nr = 1;
-};
-
-int start_server(int argc, char* argv[])
-{
-	using namespace std::literals;
-	namespace zh = zeep::http;
-
-	mmcif::CompoundFactory::init(true);
-
-	int result = 0;
-
-	po::options_description visible_options(PACKAGE_NAME " [options] input [output]");
-	visible_options.add_options()
-		("log",		po::value<std::string>(),	"Write log to this file")
-		
-		("help,h",								"Display help message")
-		("version",								"Print version")
-
-		("address",	po::value<std::string>()->default_value("0.0.0.0"),		"External address")
-		("port",	po::value<uint16_t>()->default_value(10350),			"Port to listen to")
-		("user,u",	po::value<std::string>()->default_value("www-data"),	"User to run the daemon")
-
-		("no-daemon,F",							"Do not fork into background" )
-
-		("verbose,v",							"verbose output")
-		;
-	
-	po::options_description hidden_options("hidden options");
-	hidden_options.add_options()
-		("command", po::value<std::string>(),	"The command to execute")
-		("debug,d",	po::value<int>(),			"Debug level (for even more verbose output)")
-		;
-
-	po::options_description cmdline_options;
-	cmdline_options.add(visible_options).add(hidden_options);
-
-	po::positional_options_description p;
-	p.add("command", 1);
-	
-	po::variables_map vm;
-	po::store(po::command_line_parser(argc, argv).options(cmdline_options).positional(p).run(), vm);
-	
-	po::notify(vm);
-	
-	// --------------------------------------------------------------------
-	
-	std::string secret;
-	if (vm.count("secret"))
-		secret = vm["secret"].as<std::string>();
-	else
-	{
-		secret = zeep::encode_base64(zeep::random_hash());
-		std::cerr << "starting with created secret " << secret << std::endl;
-	}
-
-	std::string user = vm["user"].as<std::string>();
-	std::string address = vm["address"].as<std::string>();
-	uint16_t port = vm["port"].as<uint16_t>();
-
-	zh::daemon server([&]()
-	{
-		auto s = new zeep::http::server();
-
-#if DEBUG
-		s->set_template_processor(new zeep::http::file_based_html_template_processor("docroot"));
-#else
-		s->set_template_processor(new zeep::http::rsrc_based_html_template_processor());
-#endif
-		s->add_controller(new tortoize_rest_controller());
-		s->add_controller(new tortoize_html_controller());
-		return s;
-	}, PACKAGE_NAME );
-
-	std::string command = vm["command"].as<std::string>();
-
-	if (command == "start")
-	{
-		std::cout << "starting server at http://" << address << ':' << port << '/' << std::endl;
-
-		if (vm.count("no-daemon"))
-			result = server.run_foreground(address, port);
-		else
-			result = server.start(address, port, 2, 2, user);
-		// server.start(vm.count("no-daemon"), address, port, 2, user);
-		// // result = daemon::start(vm.count("no-daemon"), port, user);
-	}
-	else if (command == "stop")
-		result = server.stop();
-	else if (command == "status")
-		result = server.status();
-	else if (command == "reload")
-		result = server.reload();
-	else
-	{
-		std::cerr << "Invalid command" << std::endl;
-		result = 1;
-	}
-
-	return result;
-}
-#endif
-
-// --------------------------------------------------------------------
-
-int pr_main(int argc, char* argv[])
-{
-	using namespace std::literals;
-
-#if WEBSERVICE
-	if (argc > 2 and argv[1] == "server"s)
-		return start_server(argc - 1, argv + 1);
-#endif
-
-	po::options_description visible_options(fs::path(argv[0]).filename().string() + " [options] input [output]");
-	visible_options.add_options()
-		("log",		po::value<std::string>(),	"Write log to this file")
-		
-		("dict",	po::value<std::vector<std::string>>(),
-												"Dictionary file containing restraints for residues in this specific target, can be specified multiple times.")
-
-		("help,h",								"Display help message")
-		("version",								"Print version")
-
-		("verbose,v",							"verbose output")
-		;
-	
-	po::options_description hidden_options("hidden options");
-	hidden_options.add_options()
-		("xyzin",	po::value<std::string>(),	"coordinates file")
-		("output",	po::value<std::string>(),	"Output to this file")
-		("debug,d",	po::value<int>(),			"Debug level (for even more verbose output)")
-		("build",	po::value<std::string>(),	"Build a binary data table")
-		;
-
-	po::options_description cmdline_options;
-	cmdline_options.add(visible_options).add(hidden_options);
-
-	po::positional_options_description p;
-	p.add("xyzin", 1);
-	p.add("output", 1);
-	
-	po::variables_map vm;
-	po::store(po::command_line_parser(argc, argv).options(cmdline_options).positional(p).run(), vm);
-	
-	po::notify(vm);
-
-	// --------------------------------------------------------------------
-
-	if (vm.count("version"))
-	{
-		write_version_string(std::cout, vm.count("verbose"));
-		exit(0);
-	}
-
-	if (vm.count("help"))
-	{
-		std::cout << visible_options << std::endl
-			 << std::endl
-			 << R"(Tortoize validates protein structure models by checking the 
-Ramachandran plot and side-chain rotamer distributions. Quality
-Z-scores are given at the residue level and at the model level 
-(ramachandran-z and torsions-z). Higher scores are better. To compare 
-models or to describe the reliability of the model Z-scores jackknife-
-based standard deviations are also reported (ramachandran-jackknife-sd 
-and torsion-jackknife-sd).
-
-References: 
-- Sobolev et al. A Global Ramachandran Score Identifies Protein 
-  Structures with Unlikely Stereochemistry, Structure (2020),
-  DOI: https://doi.org/10.1016/j.str.2020.08.005
-- Van Beusekom et al. Homology-based loop modeling yields more complete
-  crystallographic  protein structures, IUCrJ (2018),
-  DOI: https://doi.org/10.1107/S2052252518010552
-- Hooft et al. Objectively judging the quality of a protein structure
-  from a Ramachandran plot, CABIOS (1993),
-  DOI: https://doi.org/10.1093/bioinformatics/13.4.425 
-)" << std::endl
-			 << std::endl;
-		exit(0);
-	}
-	
-	if (vm.count("build"))
-	{
-		buildDataFile(vm["build"].as<std::string>());
-		exit(0);
-	}
-
-	if (vm.count("xyzin") == 0)
-	{
-		std::cerr << "Input file not specified" << std::endl;
-		exit(1);
-	}
-
-	cif::VERBOSE = vm.count("verbose") != 0;
-	if (vm.count("debug"))
-		cif::VERBOSE = vm["debug"].as<int>();
-
-	if (vm.count("log"))
-	{
-		if (not vm.count("output"))
-		{
-			std::cerr << "If you specify a log file, you should also specify an output file" << std::endl;
-			exit(1);
-		}
-
-		std::string logFile = vm["log"].as<std::string>();
-		
-		// open the log file
-		int fd = open(logFile.c_str(), O_CREAT|O_RDWR, 0644);
-		if (fd < 0)
-			throw std::runtime_error("Opening log file " + logFile + " failed: " + strerror(errno));
-	
-		// redirect stdout and stderr to the log file
-		dup2(fd, STDOUT_FILENO);
-		dup2(fd, STDERR_FILENO);
-		close(fd);
-	}
-
-	if (vm.count("dict"))
-	{
-		for (auto dict: vm["dict"].as<std::vector<std::string>>())
-			mmcif::CompoundFactory::instance().pushDictionary(dict);
-	}
-
-	// --------------------------------------------------------------------
-	
 	json data{
 		{ "software",
 			{
@@ -1370,10 +1018,13 @@ References:
 
 	// --------------------------------------------------------------------
 
-	mmcif::File f(vm["xyzin"].as<std::string>());
+	cif::file f = cif::pdb::read(xyzin);
+
+	if (f.empty())
+		throw std::runtime_error("Invalid or empty mmCIF/PDB file");
 
 	std::set<uint32_t> models;
-	for (auto r: f.data()["atom_site"])
+	for (auto r: f.front()["atom_site"])
 	{
 		if (not r["pdbx_PDB_model_num"].empty())
 			models.insert(r["pdbx_PDB_model_num"].as<uint32_t>());
@@ -1384,61 +1035,10 @@ References:
 
 	for (auto model: models)
 	{
-		Structure structure(f, model);
+		cif::mm::structure structure(f, model);
 
 		data["model"][std::to_string(model)] = calculateZScores(structure);
 	}
 
-	if (vm.count("output"))
-	{
-		std::ofstream of(vm["output"].as<std::string>());
-		if (not of.is_open())
-		{
-			std::cerr << "Could not open output file" << std::endl;
-			exit(1);
-		}
-		of << data;
-	}
-	else
-		std::cout << data << std::endl;
-	
-	return 0;
-}
-
-// --------------------------------------------------------------------
-
-// recursively print exception whats:
-void print_what (const std::exception& e)
-{
-	std::cerr << e.what() << std::endl;
-	try
-	{
-		std::rethrow_if_nested(e);
-	}
-	catch (const std::exception& nested)
-	{
-		std::cerr << " >> ";
-		print_what(nested);
-	}
-}
-
-#if not defined(TORTOIZE_TEST_NO_MAIN)
-
-int main(int argc, char* argv[])
-{
-	int result = -1;
-	
-	try
-	{
-		result = pr_main(argc, argv);
-	}
-	catch (std::exception& ex)
-	{
-		print_what(ex);
-		exit(1);
-	}
-
-	return result;
-}
-
-#endif
\ No newline at end of file
+	return data;
+}
\ No newline at end of file
